--[[
	Copyright (C) Udorn (Blackhand)
--]]

lootlog = lootlog or {}

lootlog.LootLog = LibStub("AceAddon-3.0"):NewAddon("LootLog", "AceEvent-3.0", "AceConsole-3.0", "AceHook-3.0")
local L = lootlog.Locale.GetInstance()
local log = lootlog.Debug:new("LootLog")

local AceConfigDialog = LibStub("AceConfigDialog-3.0")

local APP_NAME = "LootLog"

local order = 1

local SCREEN_HEIGHT = GetScreenHeight()
local SCREEN_WIDTH = GetScreenWidth()

local PLAYER = 1
local ITEM = 2
local DICE = 3

local function _NextOrder()
	order = order + 1
	return order
end

local DEFAULT_CONFIG = {
	profile = {
		backgroundColor = {0.282, 0.239, 0.541, 0.62},
		lostBackgroundColor = {0.976, 0.114, 0.112, 1},
		lostNeedOnly = true,
		wonBackgroundColor = {0.31, 0.91, 0.216, 1},
		wonNeedOnly = true,
		fadeoutTime = 30,
		showClass = true,
		minQuality = 2,
		windowPosition = {
			point = "TOPLEFT",
			relativePoint = "TOPLEFT", 
			x = SCREEN_WIDTH / 15, 
			y = -(SCREEN_HEIGHT / 3)
		}
	}
}

local OPTIONS = {
	name = "LootLog",
	handler = lootlog.LootLog,
	type = 'group',
	childGroups = "tab",
	args = {
		locked = {
			type = 'toggle',
			width = "half",
			name = L["Locked"],
			desc = L["Locks the window"],
			order = _NextOrder(),
			set = function(info, value) info.handler.db.profile.locked = value; end,
			get = function(info) return info.handler.db.profile.locked end,
		},
		test = {
			type = 'toggle',
			name = L["Test"],
			desc = L["Activates the test mode for moving the window and modifying the layout"],
			order = _NextOrder(),
			set = function(info, value) info.handler.db.profile.testMode = value; info.handler.lootWindow:TestModeUpdate() end,
			get = function(info) return info.handler.db.profile.testMode end,
		},
	 	main = {
	 	type="group",
	 	name = L["View"],
	 	order = _NextOrder(),
	 	args = {
		showClass = {
			type = 'toggle',
			name = L["Show class"],
			desc = L["Adds the class to the player name"],
			order = _NextOrder(),
			set = function(info, value) info.handler.db.profile.showClass = value end,
			get = function(info) return info.handler.db.profile.showClass end,
		},
		fadeoutTime = {
			type = 'range',
			name = L["Fadeout time"],
			desc = L["Number of seconds an item is still visible after it has been won"],
			order = _NextOrder(),
			set = function(info, value) info.handler.db.profile.fadeoutTime = value end,
			get = function(info) return info.handler.db.profile.fadeoutTime end,
			min = 1,
			max = 60,
			step = 1,
		},
		qualityNewLine = {
			type = 'description',
			name = "",
			order = _NextOrder(),
		},
		minQuality = {
			type = "select",
			name = L["Minimum quality"],
			desc = L["Only show rolls for items with at least the selected quality"],
			order = _NextOrder(),
			set = function(info, value) info.handler.db.profile.minQuality = value; info.handler:Update() end,
			get = function(info) return info.handler.db.profile.minQuality end,
			values = {[0] = ITEM_QUALITY0_DESC, [1] = ITEM_QUALITY1_DESC, [2] = ITEM_QUALITY2_DESC, [3] = ITEM_QUALITY3_DESC, [4] = ITEM_QUALITY4_DESC, [5] = ITEM_QUALITY5_DESC}
		},
--		layoutHeader = {
--			type = 'header',
--			name = L["Colors"],
--			order = _NextOrder(),
--		},
		wonNewLine = {
			type = 'description',
			name = "\n",
			order = _NextOrder(),
		},
		wonBackgroundColor = {
			type = 'color',
			name = L["Win Background"],
			desc = L["Background color for rolls won"],
			order = _NextOrder(),
			set = function(info, ...) info.handler.db.profile.wonBackgroundColor = {...}; end,
			get = function(info) return unpack(info.handler.db.profile.wonBackgroundColor) end,
			hasAlpha = true
		},
		wonNeedOnly = {
			type = 'toggle',
			name = L["Need only"],
			desc = L["Uses the given background color only for need rolls"],
			order = _NextOrder(),
			set = function(info, value) info.handler.db.profile.wonNeedOnly = value end,
			get = function(info) return info.handler.db.profile.wonNeedOnly end,
		},
		lostNewLine = {
			type = 'description',
			name = "",
			order = _NextOrder(),
		},
		lostBackgroundColor = {
			type = 'color',
			name = L["Loss Background"],
			desc = L["Background color for rolls lost"],
			order = _NextOrder(),
			set = function(info, ...) info.handler.db.profile.lostBackgroundColor = {...}; end,
			get = function(info) return unpack(info.handler.db.profile.lostBackgroundColor) end,
			hasAlpha = true
		},
		lostNeedOnly = {
			type = 'toggle',
			name = L["Need only"],
			desc = L["Uses the given background color only for need rolls"],
			order = _NextOrder(),
			set = function(info, value) info.handler.db.profile.lostNeedOnly = value end,
			get = function(info) return info.handler.db.profile.lostNeedOnly end,
		},
		backgroundColorNewLine = {
			type = 'description',
			name = "",
			order = _NextOrder(),
		},
		backgroundColor = {
			type = 'color',
			name = L["Background"],
			desc = L["General background color"],
			order = _NextOrder(),
			set = function(info, ...) info.handler.db.profile.backgroundColor = {...}; end,
			get = function(info) return unpack(info.handler.db.profile.backgroundColor) end,
			hasAlpha = true
		},
		}
		},
--		profile = {
--			type = "group",
--			name = "Profile",
--			args = {}
--		}
	},
}

local function _StripItemName(name)
	return GetItemInfo(name)
end

local function _StripDice(dice)
	-- too lazy to fix the regexp
	local s = string.gsub(dice, "%-", "")
	return strtrim(s)
end

local function _GetRaidSequence(format)
--	LOOT_ROLL_WON_NO_SPAM_DE = "%1$s won: %3$s |cff818181(Disenchant - %2$d)|r";
--	LOOT_ROLL_WON_NO_SPAM_GREED = "%1$s won: %3$s |cff818181(Greed - %2$d)|r";
--	LOOT_ROLL_WON_NO_SPAM_NEED = "%1$s won: %3$s |cff818181(Need - %2$d)|r";
	local playerIdx = strfind(format, "%%1%$s")
	local itemIdx = strfind(format, "%%3%$s")
	local diceIdx = strfind(format, "%%2%$d")
	if (playerIdx < itemIdx) then
		if (playerIdx < diceIdx) then
			if (itemIdx < diceIdx) then
				return {PLAYER, ITEM, DICE}
			end
			return {PLAYER, DICE, ITEM}
		end
		return {DICE, PLAYER, ITEM}
	elseif (playerIdx < diceIdx) then
		return {ITEM, PLAYER, DICE}
	elseif (diceIdx < itemIdx) then
		return {DICE, ITEM, PLAYER}
	end
	return {ITEM, DICE, PLAYER}
end

local function _GetRaidSelfSequence(format)
	--	LOOT_ROLL_YOU_WON_NO_SPAM_DE = "You won: %2$s |cff818181(Disenchant - %1$d)|r";
	--	LOOT_ROLL_YOU_WON_NO_SPAM_GREED = "You won: %2$s |cff818181(Greed - %1$d)|r";
	--	LOOT_ROLL_YOU_WON_NO_SPAM_NEED = "You won: %2$s |cff818181(Need - %1$d)|r";
	local itemIdx = strfind(format, "%%2%$s")
	local diceIdx = strfind(format, "%%1%$d")
	if (itemIdx < diceIdx) then
		return {ITEM, DICE}
	end
	return {DICE, ITEM}
end

local function _GetRaidSelfResults(sequence, match1, match2)
	if (sequence[1] == DICE) then
		return _StripDice(match1), _StripItemName(match2)
	end
	return _StripDice(match2), _StripItemName(match1)
end

local function _GetRaidResults(sequence, match1, match2, match3)
	if (sequence[1] == DICE) then
		if (sequence[2] == ITEM) then
			return _StripDice(match1), _StripItemName(match2), match3
		end
		return _StripDice(match1), match3, match2
	elseif (sequence[1] == ITEM) then
		if (sequence[2] == DICE) then
			return _StripDice(match2), _StripItemName(match1), match3
		end
		return match3, _StripItemName(match1), match2
	elseif (sequence[2] == DICE) then
		return _StripDice(match2), _StripItemName(match3), match1
	end
	return _StripDice(match3), _StripItemName(match2), match1
end

local function _InitPatterns(self)
	log:Debug("_InitPatterns")
--	LOOT_ROLL_ROLLED_DE = "Disenchant Roll - %d for %s by %s";
--	LOOT_ROLL_ROLLED_GREED = "Greed Roll - %d for %s by %s";
--	LOOT_ROLL_ROLLED_NEED = "Need Roll - %d for %s by %s";
--  LOOT_ROLL_NEED_SELF = "You have selected Need for: %s";
--	LOOT_ROLL_GREED_SELF = "You have selected Greed for: %s";
--  LOOT_ROLL_DISENCHANT_SELF = "You have selected Disenchant for: %s";
--	LOOT_ROLL_PASSED_SELF = "You passed on: %s";
--	LOOT_ROLL_PASSED_SELF_AUTO = "You automatically passed on: %s because you cannot loot that item.";
--	LOOT_ROLL_YOU_WON = "You won: %s";
--	LOOT_ROLL_WON = "%s won: %s";
--  LOOT_ROLL_ALL_PASSED = "Everyone passed on: %s";

--	LOOT_ROLL_WON_NO_SPAM_DE = "%1$s won: %3$s |cff818181(Disenchant - %2$d)|r";
--	LOOT_ROLL_WON_NO_SPAM_GREED = "%1$s won: %3$s |cff818181(Greed - %2$d)|r";
--	LOOT_ROLL_WON_NO_SPAM_NEED = "%1$s won: %3$s |cff818181(Need - %2$d)|r";
--	LOOT_ROLL_YOU_WON_NO_SPAM_DE = "You won: %2$s |cff818181(Disenchant - %1$d)|r";
--	LOOT_ROLL_YOU_WON_NO_SPAM_GREED = "You won: %2$s |cff818181(Greed - %1$d)|r";
--	LOOT_ROLL_YOU_WON_NO_SPAM_NEED = "You won: %2$s |cff818181(Need - %1$d)|r";
--	LOOT_ROLL_ROLLED_NEED_ROLE_BONUS = "Need Roll - %d for %s by %s + Role Bonus";

	self.raidSelfNeed = string.gsub(string.gsub(LOOT_ROLL_YOU_WON_NO_SPAM_NEED, "%%1%$d", "(.*)"), "%%2%$s", "(.*)")
	self.raidSelfNeedSequence = _GetRaidSelfSequence(LOOT_ROLL_YOU_WON_NO_SPAM_NEED)
	self.raidSelfGreed = string.gsub(string.gsub(LOOT_ROLL_YOU_WON_NO_SPAM_GREED, "%%1%$d", "(.*)"), "%%2%$s", "(.*)")
	self.raidSelfGreedSequence = _GetRaidSelfSequence(LOOT_ROLL_YOU_WON_NO_SPAM_GREED)
	self.raidSelfDisenchant = string.gsub(string.gsub(LOOT_ROLL_YOU_WON_NO_SPAM_DE, "%%1%$d", "(.*)"), "%%2%$s", "(.*)")
	self.raidSelfDisenchantSequence = _GetRaidSelfSequence(LOOT_ROLL_YOU_WON_NO_SPAM_DE)
	self.raidNeed = string.gsub(string.gsub(LOOT_ROLL_WON_NO_SPAM_NEED, "%%2%$d", "(.*)"), "%%%d%$s", "(.*)")
	self.raidNeedSequence = _GetRaidSequence(LOOT_ROLL_WON_NO_SPAM_NEED)
	self.raidGreed = string.gsub(string.gsub(LOOT_ROLL_WON_NO_SPAM_GREED, "%%2%$d", "(.*)"), "%%%d%$s", "(.*)")
	self.raidGreedSequence = _GetRaidSequence(LOOT_ROLL_WON_NO_SPAM_GREED)
	self.raidDisenchant = string.gsub(string.gsub(LOOT_ROLL_WON_NO_SPAM_DE, "%%2%$d", "(.*)"), "%%%d%$s", "(.*)")
	self.raidDisenchantSequence = _GetRaidSequence(LOOT_ROLL_WON_NO_SPAM_DE)
	
	self.raidNeedRole = string.gsub(string.gsub(string.gsub(LOOT_ROLL_ROLLED_NEED_ROLE_BONUS, "%%d", "(.*)"), "%%s", "(.*)"), "%+", "%%+")
	
	self.disenchantRollPattern = string.gsub(string.gsub(LOOT_ROLL_ROLLED_DE, "%%d", "(.*)"), "%%s", "(.*)")
	self.greedRollPattern = string.gsub(string.gsub(LOOT_ROLL_ROLLED_GREED, "%%d", "(.*)"), "%%s", "(.*)")
	self.needRollPattern = string.gsub(string.gsub(LOOT_ROLL_ROLLED_NEED, "%%d", "(.*)"), "%%s", "(.*)")
	
	self.selfNeedPattern = string.gsub(LOOT_ROLL_NEED_SELF, "%%s", "(.*)")
	self.selfGreedPattern = string.gsub(LOOT_ROLL_GREED_SELF, "%%s", "(.*)")
	self.selfDisenchantPattern = string.gsub(LOOT_ROLL_DISENCHANT_SELF, "%%s", "(.*)")
	self.selfPassedPattern = string.gsub(LOOT_ROLL_PASSED_SELF, "%%s", "(.*)")
	self.selfAutoPassedPattern = string.gsub(LOOT_ROLL_PASSED_SELF_AUTO, "%%s", "(.*)")
	
	self.allPassedPattern = string.gsub(LOOT_ROLL_ALL_PASSED, "%%s", "(.*)")
		
	self.selfWonPattern = string.gsub(LOOT_ROLL_YOU_WON, "%%s", "(.*)")
	self.wonPattern = string.gsub(LOOT_ROLL_WON, "%%s", "(.*)")
end
 
function lootlog.LootLog:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("LootLogDb", DEFAULT_CONFIG, "Default")
	OPTIONS.args.profile = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)
	self.db.profile.testMode = nil -- just in case
	
	self.db.RegisterCallback(self, "OnProfileChanged", "ProfileUpdate")
	self.db.RegisterCallback(self, "OnProfileCopied", "ProfileUpdate")
	self.db.RegisterCallback(self, "OnProfileReset", "ProfileUpdate")
	
	_InitPatterns(self)
end

function lootlog.LootLog:OnEnable()
	self.lootWindow = lootlog.LootWindow:new()
	self.playerName = GetUnitName("player")
	
	LibStub("AceConfig-3.0"):RegisterOptionsTable(APP_NAME, OPTIONS)
	self.configFrame = AceConfigDialog:AddToBlizOptions(APP_NAME)
	self:RegisterChatCommand("lootlog", "ChatCommand")
		
	self:RegisterEvent("CHAT_MSG_LOOT")
	self:RegisterEvent("START_LOOT_ROLL")
	self:RegisterEvent("CANCEL_LOOT_ROLL")
	self:RegisterEvent("CONFIRM_DISENCHANT_ROLL")
	self:RegisterEvent("CONFIRM_LOOT_ROLL")

	--self:CHAT_MSG_LOOT("", "Wurf für Bedarf: 197 für |cffa335ee|Hitem:78485:0:0:0:0:0:0:432663696:85:0|h[Schlund des Drachenlords]|h|r von Alrock + Rollenbonus")
end

function lootlog.LootLog:CHAT_MSG_LOOT(evt, msg, id)
	log:Debug("CHAT_MSG_LOOT msg [%s] id [%s]", msg, id)
	
	local itemName = string.match(msg, self.selfNeedPattern)
	if (itemName) then
		log:Debug("1")
		self.lootWindow:SetPlayerRoll(lootlog.LootWindow.NEED, _StripItemName(itemName))
		return
	end
	local itemName = string.match(msg, self.selfGreedPattern)
	if (itemName) then
		log:Debug("2")
		self.lootWindow:SetPlayerRoll(lootlog.LootWindow.GREED, _StripItemName(itemName))
		return
	end
	local itemName = string.match(msg, self.selfDisenchantPattern)
	if (itemName) then
		log:Debug("3")
		self.lootWindow:SetPlayerRoll(lootlog.LootWindow.DISENCHANT, _StripItemName(itemName))
		return
	end
	local itemName = string.match(msg, self.selfPassedPattern)
	if (itemName) then
		log:Debug("4")
		self.lootWindow:SetPlayerRoll(lootlog.LootWindow.PASSED, _StripItemName(itemName))
		return
	end
	local itemName = string.match(msg, self.selfAutoPassedPattern)
	if (itemName) then
		log:Debug("5")
		self.lootWindow:SetPlayerRoll(lootlog.LootWindow.PASSED, _StripItemName(itemName))
		return
	end

	local match1, match2 = string.match(msg, self.raidSelfNeed)
	if (match1) then
		log:Debug("6")
		self.lootWindow:SetRoll(lootlog.LootWindow.NEED, _GetRaidSelfResults(self.raidSelfNeedSequence, match1, match2), self.playerName)
		return
	end
	local match1, match2 = string.match(msg, self.raidSelfGreed)
	if (match1) then
		log:Debug("7")
		self.lootWindow:SetRoll(lootlog.LootWindow.GREED, _GetRaidSelfResults(self.raidSelfGreedSequence, match1, match2), self.playerName)
		return
	end
	local match1, match2 = string.match(msg, self.raidSelfDisenchant)
	if (match1) then
		log:Debug("8")
		self.lootWindow:SetRoll(lootlog.LootWindow.DISENCHANT, _GetRaidSelfResults(self.raidSelfDisenchantSequence, match1, match2), self.playerName)
		return
	end
	local match1, match2, match3 = string.match(msg, self.raidNeed)
	if (match1) then
		log:Debug("9")
		self.lootWindow:SetRoll(lootlog.LootWindow.NEED, _GetRaidResults(self.raidNeedSequence, match1, match2, match3))
		return
	end
	local match1, match2, match3 = string.match(msg, self.raidGreed)
	if (match1) then
		log:Debug("10")
		self.lootWindow:SetRoll(lootlog.LootWindow.GREED, _GetRaidResults(self.raidGreedSequence, match1, match2, match3))
		return
	end
	local match1, match2, match3 = string.match(msg, self.raidDisenchant)
	if (match1) then
		log:Debug("11")
		self.lootWindow:SetRoll(lootlog.LootWindow.DISENCHANT, _GetRaidResults(self.raidDisenchantSequence, match1, match2, match3))
		return
	end

	local itemName = string.match(msg, self.allPassedPattern)
	if (itemName) then
		log:Debug("12")
		self.lootWindow:SetWinner(_StripItemName(itemName), nil)
		return
	end

	local itemName = string.match(msg, self.selfWonPattern)
	if (itemName) then
		log:Debug("13")
		self.lootWindow:SetWinner(_StripItemName(itemName), self.playerName)
		return
	end
	local playerName, itemName = string.match(msg, self.wonPattern)
	if (playerName) then
		log:Debug("14")
		self.lootWindow:SetWinner(_StripItemName(itemName), playerName)
		return
	end

	-- "Need Roll - %d for %s by %s + Role Bonus";
	local dice, itemName, playerName = string.match(msg, self.raidNeedRole)
	if (dice) then
		log:Debug("14.1")
		self.lootWindow:SetRoll(lootlog.LootWindow.NEED, _StripDice(dice), _StripItemName(itemName), playerName)
		return
	end

	local dice, itemName, playerName = string.match(msg, self.disenchantRollPattern)
	if (dice) then
		log:Debug("15")
		self.lootWindow:SetRoll(lootlog.LootWindow.DISENCHANT, _StripDice(dice), _StripItemName(itemName), playerName)
		return
	end
	local dice, itemName, playerName = string.match(msg, self.greedRollPattern)
	if (dice) then
		log:Debug("16")
		self.lootWindow:SetRoll(lootlog.LootWindow.GREED, _StripDice(dice), _StripItemName(itemName), playerName)
		return
	end
	local dice, itemName, playerName = string.match(msg, self.needRollPattern)
	if (dice) then
		log:Debug("17")
		self.lootWindow:SetRoll(lootlog.LootWindow.NEED, _StripDice(dice), _StripItemName(itemName), playerName)
		return
	end
	log:Debug("exit without match")
end

function lootlog.LootLog:ProfileUpdate(event)
	log:Debug("ProfileUpdate event [%s]", event)
	self.lootWindow:ProfileUpdate()
end

function lootlog.LootLog:Update()
	self.lootWindow:TestModeUpdate()
end


function lootlog.LootLog:ChatCommand(msg)
	log:Debug("msg [%s]", msg)
	if ("debug" == msg) then
		if (log:IsDebugging()) then
			print("Turning off debugging")
			log:SetDebugging(nil)
		else
			print("Turning on debugging")
			log:SetDebugging(true)
		end
	elseif ("history" == msg) then
		print("Printing history")
		log:PrintHistory()
	elseif ("clear" == msg) then
		print("Clearing history")
		log:ClearHistory()
	else
		InterfaceOptionsFrame_OpenToCategory(self.configFrame)
	end
end

function lootlog.LootLog:START_LOOT_ROLL(evt, rollId, rollTime)
	log:Debug("START_LOOT_ROLL rollId [%s] rollTime [%s]", rollId, rollTime)
	local texture, name, count, quality = GetLootRollItemInfo(rollId)
	local itemLink = GetLootRollItemLink(rollId)
	if (name and itemLink and quality >= self.db.profile.minQuality) then
		log:Debug("name [%s] quality [%s]", name, quality)
		self.lootWindow:AddRoll(rollId, name, itemLink, quality)
	end
end

function lootlog.LootLog:CANCEL_LOOT_ROLL(evt, rollId)
	log:Debug("CANCEL_LOOT_ROLL rollId [%s]", rollId)
end

function lootlog.LootLog:CONFIRM_DISENCHANT_ROLL(evt, rollId)
	log:Debug("CONFIRM_DISENCHANT_ROLL rollId [%s]", rollId)
end

function lootlog.LootLog:CONFIRM_LOOT_ROLL(evt, rollId, roll)
	log:Debug("CONFIRM_LOOT_ROLL rollId [%s] roll [%s]", rollId, roll)
end
